//**********************************************************
//* Super Mario Kart MSU-1 Audio Hack
//*
//* Copyright qwertymodo, 2017
//* 
//* Assembler: bass v14
//*  NOTE: rom needs to be expanded before applying the asm (e.g., via Lunar Expand to 1MB)
//*   bass -d input=smk.sfc -d STAR_RESUME -o smk_msu_v13_Star_Resume.sfc -create mk_msu.asm
//*   bass -d input=smk.sfc -o smk_msu_v13_NoStar_Resume.sfc -create mk_msu.asm
//*
//* Song List:
//*   01: Race Fanfare
//*   05: Final Lap Notice
//*   08: Super Star Theme
//*   09: New Record
//*   10: No Record
//*   11: Ranked Out
//*   12: Mario's Rank
//*   13: Luigi's Rank
//*   14: Bowser's Rank
//*   15: Peach's Rank
//*   16: DK's Rank
//*   17: Koopa's Rank
//*   18: Toad's Rank
//*   19: Yoshi's Rank
//*   20: Game Over
//*
//*   32: Title Screen
//*   35: Kart Select
//*   36: Staff Roll
//*   38: Mario Circuit**
//*   41: Donut Plains**
//*   44: Choco Island**
//*   47: Koopa Beach**
//*   50: Vanilla Lake**
//*   53: Ghost Valley**
//*   56: Bowser Castle**
//*   59: Battle Mode**
//*   62: Tournament Win
//*   65: Tournament Lose
//*   71: Rainbow Road**
//*
//*   80: Mario Circuit 3*
//*   82: Ghost Valley 2*
//*   84: Donut Plains 2*
//*   86: Bowser Castle 2*
//*   88: Vanilla Lake 2*
//*   90: Rainbow Road*
//*   92: Koopa Beach 2*
//*   94: Mario Circuit 1*
//*   96: Ghost Valley 3*
//*   98: Bowser Castle 3*
//*  100: Choco Island 2*
//*  102: Donut Plains 3*
//*  104: Vanilla Lake 1*
//*  106: Koopa Beach 1*
//*  108: Mario Circuit 4*
//*  110: Mario Circuit 2*
//*  112: Ghost Valley 1*
//*  114: Bowser Castle 1*
//*  116: Choco Island 1*
//*  118: Donut Plains 1*
//*
//*  [*]  Song number +1 for fast version
//*  [**] Same song for all songs, +1 for fast version
//**********************************************************

arch snes.cpu

define hirom()

if {defined STAR_RESUME} {
    define CHECKSUM($EE81)
} else {
    define CHECKSUM($F9EF)
}

include "snes_utils.inc"
include "msu-1_defs.inc"



define REG_CURRENT_VOLUME($7e1f60)
define REG_TARGET_VOLUME($7e1f61)
define REG_CURRENT_SONG($7e1f62)
define REG_CURRENT_SONG_SHORT($1f62)
define REG_NOLOOP_SONG($7e1f64)


define REG_CURRENT_TRACK($7E0124)

define VAL_VOLUME_INCREMENT(#$0e)
define VAL_VOLUME_DECREMENT(#$0e)
define VAL_VOLUME_MUTE(#$00)
define VAL_VOLUME_HALF(#$30)
define VAL_VOLUME_FULL(#$Ff)


scope {

seek($809FA0)
    jsl pause_halfVolume
    nop	
assert_end($809FA5)

seek($809FA6)
    jsl pause_fullVolume
    nop
assert_end($809FAB)

seek($81F4D3)
    jsl mute_giveup
    nop
assert_end($81F4D8)

seek($81F432)
    jsl store_bank
    nop
    nop
assert_end($81F438)

seek($8097A3)
    jsr msu_short_jump
assert_end($8097A6)

seek($C09615)
    jsr msu_short_jump
assert_end($C0961a)




seek($80ffa0)
msu_short_jump:
    jsl msu_main
    rts
assert_end($80ffa5)
}






scope {





seek($C80000)
pause_halfVolume:
   lda #$0080
   tsb $40
   php 
   sep #$20
   lda #$30
   sta {REG_TARGET_VOLUME}
   plp
   rtl
 
mute_giveup:    
   and #$8080
   sta $42
   php 
   sep #$20
   lda #$00
   sta {REG_TARGET_VOLUME}
   plp
   rtl

pause_fullVolume:
   lda #$0080
   trb $40
   php 
   sep #$20
   lda #$FF
   sta {REG_TARGET_VOLUME}
   plp
   rtl
    

store_bank:
    stx {REG_CURRENT_SONG_SHORT}
    and #$00FF
    ldx #$C000
    rtl



msu_main:
    pha
    lda $2002
    cmp #$53
    beq +
    jmp spc_continue
+;  pla
    pha
    cmp #$00
    bne +
    jmp do_fade
+;  cmp #$80
    bne + 
    jmp do_fade

// Command $1D: Mute
+;  cmp #$1D
    bne +
    lda #$00
    sta {REG_TARGET_VOLUME}
    sta {REG_MSU_VOLUME}
    bra do_fade

// Command $1E: Fade Out    
+;  cmp #$1E
    bne +
    lda #$00
    sta {REG_TARGET_VOLUME}
    bra do_fade

// Command $1F: Fade In
+;  cmp #$1F
    bne +
    lda #$FF
    sta {REG_TARGET_VOLUME}
    bra do_fade

// Command $04/$16: Play Song (Normal Speed)
+;  cmp #$04
    beq +
    cmp #$16
    bne ++
+;  brl play_song_normal

// Command $06/07/15: Play Song (Fast Speed)
+;  cmp #$06
    beq +
    cmp #$07
    beq +
    cmp #$15
    bne ++
+;  brl play_song_fast

// Command $08: Invincible
+;  cmp #$08
    bne ++
    pha
if {defined STAR_RESUME} {
    lda {REG_MSU_STATUS}
    bit #$30 
    beq +
    and #$07
    cmp #$02
    bcc +
    lda {FLAG_MSU_RESUME}
    sta {REG_MSU_CONTROL}
}
+;  pla
    jsr check_song_exists
    bcs new 
    jmp spc_continue
    new:
    jmp play_loop

// Commands >= $20: Send to SPC normally
+;  cmp #$20
    bcc +
    jmp spc_continue
    
// Ranks
+;  jsr check_song_exists
    bcs +
    jmp spc_continue
+;  cmp #$0C
    bcs +
    jmp play_once
+;  cmp #$14
    bcc +
    jmp play_once
+;  jmp play_loop


do_fade:
    lda {REG_NOLOOP_SONG}
    beq +
    jmp spc_continue
+;  lda {REG_TARGET_VOLUME}
    cmp {REG_CURRENT_VOLUME}  
    bne +
    jmp spc_continue
+;  cmp {VAL_VOLUME_HALF}
    beq halfVolume
    cmp {VAL_VOLUME_FULL}
    beq fullVolume
   
//zero volume
		lda {REG_CURRENT_VOLUME}
		sec
		sbc {VAL_VOLUME_DECREMENT}
		cmp #$10
		bcs +
		lda {REG_TARGET_VOLUME}
+;	sta $2006
		sta {REG_CURRENT_VOLUME}
		jmp spc_continue

halfVolume:
		lda {REG_CURRENT_VOLUME}
		sec
		sbc {VAL_VOLUME_DECREMENT}
		cmp {VAL_VOLUME_HALF}
		bcs +
		lda {REG_TARGET_VOLUME}	
+;	sta $2006
		sta {REG_CURRENT_VOLUME}
		jmp spc_continue

fullVolume:
		lda {REG_CURRENT_VOLUME}
		clc
		adc {VAL_VOLUME_INCREMENT}
		cmp #$f0
		bcc +
		lda {REG_TARGET_VOLUME}
+;	sta $2006
		sta {REG_CURRENT_VOLUME}
		jmp spc_continue 
    



    
play_loop:
    lda {VAL_VOLUME_FULL}
    sta {REG_TARGET_VOLUME}
    sta {REG_CURRENT_VOLUME}
    sta {REG_MSU_VOLUME}
    lda #$03
    sta {REG_MSU_CONTROL}
    lda #$00
    sta {REG_NOLOOP_SONG}
    bra spc_mute

play_once:
    lda {VAL_VOLUME_FULL}
    sta {REG_TARGET_VOLUME}
    sta {REG_CURRENT_VOLUME}
    sta {REG_MSU_VOLUME}
    lda #$01
    sta {REG_MSU_CONTROL}
    sta {REG_NOLOOP_SONG}    
    bra spc_mute
    
spc_continue:
    pla
    sta {REG_APU_PORT_0}
    rep #$30
    rtl
    
spc_mute:

    pla
    lda #$1D
    sta {REG_APU_PORT_0}
    rep #$30
    rtl



spc_skip:

    pla
    rep #$30
    rtl

play_song_normal:
// Check special tracks
    lda {REG_CURRENT_SONG}
// Title Screen
    beq ++
// Select Screen
    cmp #$03
    beq ++
// Battle Mode
    cmp #$1B
    beq ++
// Tournament Win
    cmp #$1E
    beq ++
// Tournament Lose
    cmp #$21
    beq ++
// Staff Roll
    cmp #$24
    bne +
    jsr check_song_exists
    bcc spc_continue
    bra play_once
// Check Track-Specific Songs
+;  lda {REG_CURRENT_TRACK}
    asl
    clc
    adc #$50
    jsr check_song_exists
    bcs play_loop
// Check Shared Songs
    lda {REG_CURRENT_SONG}
+;  clc
    adc #$20
    jsr check_song_exists
    bcc spc_continue
    jmp play_loop

play_song_fast:
// Check Track-Specific Songs
    lda {REG_CURRENT_TRACK}
    asl
    clc
    adc #$51
    jsr check_song_exists
    bcc +
    brl play_loop
// Check Shared Songs
+;  lda {REG_CURRENT_SONG}
    clc
    adc #$21
    jsr check_song_exists
    bcc spc_continue
    brl play_loop


// Input: Song # (A, 8-bit)
// Output: Carry flag, set = true, clear = false
check_song_exists:
    pha
    sta {REG_MSU_TRACK_LO}
    stz {REG_MSU_TRACK_HI}
-;  lda {REG_MSU_STATUS}
    bit {FLAG_MSU_STATUS_AUDIO_BUSY}
    bne -
    bit {FLAG_MSU_STATUS_TRACK_MISSING}
    bne +
    pla
    sec
    rts
+;  pla
    clc
    rts
assert_end($D01000)
}
